package jalview.api;

import jalview.datamodel.Mapping;

//JBPComment: this is a datamodel API - so it should be in datamodel (it's a peer of SequenceI)

public interface DBRefEntryI
{
  public boolean equalRef(DBRefEntryI entry);

  /**
   * 
   * @return Source DB name for this entry
   */
  public String getSource();

  /**
   * 
   * @return Accession Id for this entry
   */
  public String getAccessionId();

  /**
   * 
   * @param accessionId
   *          Accession Id for this entry
   */
  public void setAccessionId(String accessionId);

  /**
   * 
   * @param source
   *          Source DB name for this entry
   */
  public void setSource(String source);

  /**
   * 
   * @return Source DB version for this entry
   */
  public String getVersion();

  /**
   * 
   * @param version
   *          Source DB version for this entry
   */
  public void setVersion(String version);

  /**
   * access a mapping, if present that can be used to map positions from the
   * associated dataset sequence to the DBRef's sequence frame.
   * 
   * @return null or a valid mapping.
   */
  public Mapping getMap();

  /**
   * Answers true if this object is either equivalent to, or can be 'improved'
   * by, the given entry. Specifically, answers true if
   * <ul>
   * <li>source and accession are identical</li>
   * <li>version is identical, or this version is of the format "someSource:0",
   * in which case the version for the other entry replaces it</li>
   * <li>mappings are not compared but if this entry has no mapping, replace
   * with that for the other entry</li>
   * </ul>
   * 
   * @param otherEntry
   * @return
   */
  public boolean updateFrom(DBRefEntryI otherEntry);

  /**
   * Method to distinguish between direct and indirect database references
   * 
   * primary references indicate the local sequence data directly corresponds
   * with the database record. All other references are secondary. direct
   * references indicate that part or all of the local sequence data can be
   * mapped with another sequence, enabling annotation transfer.
   * cross-references indicate the local sequence data can be corresponded to
   * some other linear coordinate system via a transformation.
   * 
   * This method is also sufficient to distinguish direct DBRefEntry mappings
   * from other relationships - e.g. coding relationships (imply a 1:3/3:1
   * mapping), but not transcript relationships, which imply a (possibly
   * non-contiguous) 1:1 mapping
   * 
   * The only way a dbref's mappings can be fully verified is via the local
   * sequence frame, so rather than use isPrimary directly, please use
   * SequenceI.getPrimaryDbRefs()
   *
   * @return true if this reference provides a primary accession for the
   *         associated sequence object
   */
  public boolean isPrimary();
}
